@@ -2,7 +2,7 @@
    standard output a file of nroff output containing the doc strings.
 
 Copyright (C) 1987, 1994, 2001, 2002, 2003, 2004, 2005, 2006, 2007,
-  2008, 2009, 2010 Free Software Foundation, Inc.
+  2008, 2009, 2010, 2011 Free Software Foundation, Inc.
 
 This file is part of GNU Emacs.
 
