@@ -1,7 +1,7 @@
 ;;; find-dired.el --- run a `find' command and dired the output
 
 ;; Copyright (C) 1992, 1994, 1995, 2000, 2001, 2002, 2003, 2004,
-;;   2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+;;   2005, 2006, 2007, 2008, 2009, 2010, 2011 Free Software Foundation, Inc.
 
 ;; Author: Roland McGrath <roland@gnu.org>,
 ;;	   Sebastian Kremer <sk@thp.uni-koeln.de>
@@ -38,8 +38,8 @@
 ;; Note -b, at least GNU find quotes spaces etc. in filenames
 ;;;###autoload
 (defcustom find-ls-option
-  (if (eq system-type 'berkeley-unix) '("-ls" . "-gilsb")
-    '("-exec ls -ld {} \\;" . "-ld"))
+  (if (eq system-type 'berkeley-unix) (purecopy '("-ls" . "-gilsb"))
+    (purecopy '("-exec ls -ld {} \\;" . "-ld")))
   "Description of the option to `find' to produce an `ls -l'-type listing.
 This is a cons of two strings (FIND-OPTION . LS-SWITCHES).  FIND-OPTION
 gives the option (or options) to `find' that produce the desired output.
@@ -49,7 +49,7 @@ LS-SWITCHES is a list of `ls' switches to tell dired how to parse the output."
   :group 'find-dired)
 
 ;;;###autoload
-(defcustom find-ls-subdir-switches "-al"
+(defcustom find-ls-subdir-switches (purecopy "-al")
   "`ls' switches for inserting subdirectories in `*Find*' buffers.
 This should contain the \"-l\" switch.
 Use the \"-F\" or \"-b\" switches if and only if you also use
@@ -60,24 +60,24 @@ them for `find-ls-option'."
 
 ;;;###autoload
 (defcustom find-grep-options
-  (if (or (eq system-type 'berkeley-unix)
+  (purecopy (if (or (eq system-type 'berkeley-unix)
 	  (string-match "solaris2" system-configuration)
 	  (string-match "irix" system-configuration))
-      "-s" "-q")
+      "-s" "-q"))
   "Option to grep to be as silent as possible.
 On Berkeley systems, this is `-s'; on Posix, and with GNU grep, `-q' does it.
 On other systems, the closest you can come is to use `-l'."
   :type 'string
   :group 'find-dired)
 
-;;;###autoload
+;; This used to be autoloaded (see bug#4387).
 (defcustom find-name-arg
-  (if read-file-name-completion-ignore-case
+  (purecopy (if read-file-name-completion-ignore-case
       "-iname"
-    "-name")
+    "-name"))
   "Argument used to specify file name pattern.
 If `read-file-name-completion-ignore-case' is non-nil, -iname is used so that
-find also ignores case. Otherwise, -name is used."
+find also ignores case.  Otherwise, -name is used."
   :type 'string
   :group 'find-dired
   :version "22.2")
@@ -291,8 +291,7 @@ Thus ARG can also contain additional grep options."
   (let ((buf (process-buffer proc))
 	(inhibit-read-only t))
     (if (buffer-name buf)
-	(save-excursion
-	  (set-buffer buf)
+	(with-current-buffer buf
 	  (let ((buffer-read-only nil))
 	    (save-excursion
 	      (goto-char (point-max))
