@@ -1,17 +1,17 @@
 ;;; page.el --- page motion commands for Emacs
 
 ;; Copyright (C) 1985, 2001, 2002, 2003, 2004, 2005,
-;;   2006, 2007 Free Software Foundation, Inc.
+;;   2006, 2007, 2008, 2009, 2010, 2011 Free Software Foundation, Inc.
 
 ;; Maintainer: FSF
 ;; Keywords: wp convenience
 
 ;; This file is part of GNU Emacs.
 
-;; GNU Emacs is free software; you can redistribute it and/or modify
+;; GNU Emacs is free software: you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
-;; the Free Software Foundation; either version 2, or (at your option)
-;; any later version.
+;; the Free Software Foundation, either version 3 of the License, or
+;; (at your option) any later version.
 
 ;; GNU Emacs is distributed in the hope that it will be useful,
 ;; but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -19,9 +19,7 @@
 ;; GNU General Public License for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with GNU Emacs; see the file COPYING.  If not, write to the
-;; Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-;; Boston, MA 02110-1301, USA.
+;; along with GNU Emacs.  If not, see <http://www.gnu.org/licenses/>.
 
 ;;; Commentary:
 
@@ -151,11 +149,12 @@ thus showing a page other than the one point was originally in."
   (save-restriction
     (widen)
     (save-excursion
-      (beginning-of-line)
       (let ((count 1)
 	    (opoint (point)))
-	(goto-char 1)
+	(goto-char (point-min))
 	(while (re-search-forward page-delimiter opoint t)
+          (if (= (match-beginning 0) (match-end 0))
+              (forward-char 1))
 	  (setq count (1+ count)))
 	(message "Page %d, line %d"
 		 count
@@ -164,5 +163,5 @@ thus showing a page other than the one point was originally in."
 ;;; Place `provide' at end of file.
 (provide 'page)
 
-;;; arch-tag: e8d7a0bd-8655-4b6e-b852-f2ee25316a1d
+;; arch-tag: e8d7a0bd-8655-4b6e-b852-f2ee25316a1d
 ;;; page.el ends here
